@@ -1,13 +1,15 @@
 /* errno.c
  *
- * Copyright (c) 1996-2001 Mike Gleason, NCEMRSoft.
+ * Copyright (c) 1996-2005 Mike Gleason, NcFTP Software.
  * All rights reserved.
  *
  */
 
-#define _libncftp_errno_c_ 1
 #include "syshdrs.h"
-	
+#ifdef PRAGMA_HDRSTOP
+#	pragma hdrstop
+#endif
+
 static const char *gErrList[kErrLast - kErrFirst + 2] = {
 	"gethostname() failed",						/* -100 */
 	"hostname does not include domain name",			/* -101 */
@@ -87,7 +89,7 @@ static const char *gErrList[kErrLast - kErrFirst + 2] = {
 	"file transfer buffer has not been allocated",			/* -175 */
 	"will not overwrite local file with older remote file",		/* -176 */
 	"will not overwrite remote file with older local file",		/* -177 */
-	"local file appears to be the same as the remote file, no transfer necessary",	/* -178 */
+	"local file appears to be the same as the remote file, download is not necessary",	/* -178 */
 	"could not get extended directory information (MLSD)",		/* -179 */
 	"could not get extended file or directory information (MLST)",	/* -180 */
 	"could not parse extended file or directory information",	/* -181 */
@@ -105,10 +107,22 @@ static const char *gErrList[kErrLast - kErrFirst + 2] = {
 	"timed out while waiting for server response",			/* -193 */
 	"data transfer timed out",					/* -194 */
 	"canceled by user",						/* -195 */
+	"no remote host was specified",					/* -196 */
+	"remote file appears to be the same as the local file, upload is not necessary",	/* -197 */
+	"data connection did not originate from same IP address as control connection to server",	/* -198 */
+	"data connection did not originate from correct port number",	/* -199 */
+	"internal error: bug in library code detected",			/* -200 */
+	"remote server does not support setting a RESTart point",	/* -201 */
+	"miscellaneous error occurred while trying to open the host", 	/* -202 */
+	"miscellaneous error occurred while trying to login to the host", /* -203 */
+	"ascii seek error",						/* -204 */
+	"you have encountered a bug that we have not fixed yet, sorry!",/* -205 */
+	"could not bind the control connection socket",			/* -206 */
+	"pathname too long (internal buffer too small to hold it)",	/* -207 */
 	NULL,								
 };
 
-int gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = 0;
+const char *gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings = 0;
 
 const char *
 FTPStrError(int e)
@@ -130,6 +144,29 @@ FTPStrError(int e)
 
 
 
+char *
+FTPStrError2(const FTPCIPtr cip, int e, char *const dst, const size_t dstsize, int eerr)
+{
+	FTPLinePtr lp;
+
+	if ((dst == NULL) || (dstsize == 0))
+		return NULL;
+
+	if ((e == eerr) && (e != kNoErr)) {
+		(void) Strncpy(dst, "Server said:", dstsize);
+		for (lp = cip->lastFTPCmdResultLL.first; lp != NULL; lp = lp->next) {
+			(void) Strncat(dst, " ", dstsize);
+			(void) Strncat(dst, lp->line, dstsize);
+		}
+	} else {
+		(void) Strncpy(dst, FTPStrError(e), dstsize);
+	}
+	return (dst);
+}	/* FTPStrError2 */
+
+
+
+
 void
 FTPPerror(const FTPCIPtr cip, const int err, const int eerr, const char *const s1, const char *const s2)
 {
@@ -137,26 +174,26 @@ FTPPerror(const FTPCIPtr cip, const int err, const int eerr, const char *const s
 		if (err == eerr) {
 			if ((s2 == NULL) || (s2[0] == '\0')) {
 				if ((s1 == NULL) || (s1[0] == '\0')) { 
-					(void) fprintf(stderr, "server said: %s\n", cip->lastFTPCmdResultStr);
+					FTPLogError(cip, kDontPerror, "server said: %s\n", cip->lastFTPCmdResultStr);
 				} else {
-					(void) fprintf(stderr, "%s: server said: %s\n", s1, cip->lastFTPCmdResultStr);
+					FTPLogError(cip, kDontPerror, "%s: server said: %s\n", s1, cip->lastFTPCmdResultStr);
 				}
 			} else if ((s1 == NULL) || (s1[0] == '\0')) { 
-				(void) fprintf(stderr, "%s: server said: %s\n", s2, cip->lastFTPCmdResultStr);
+				FTPLogError(cip, kDontPerror, "%s: server said: %s\n", s2, cip->lastFTPCmdResultStr);
 			} else {
-				(void) fprintf(stderr, "%s %s: server said: %s\n", s1, s2, cip->lastFTPCmdResultStr);
+				FTPLogError(cip, kDontPerror, "%s %s: server said: %s\n", s1, s2, cip->lastFTPCmdResultStr);
 			}
 		} else {
 			if ((s2 == NULL) || (s2[0] == '\0')) {
 				if ((s1 == NULL) || (s1[0] == '\0')) { 
-					(void) fprintf(stderr, "%s.\n", FTPStrError(cip->errNo));
+					FTPLogError(cip, kDontPerror, "%s.\n", FTPStrError(err));
 				} else {
-					(void) fprintf(stderr, "%s: %s.\n", s1, FTPStrError(cip->errNo));
+					FTPLogError(cip, kDontPerror, "%s: %s.\n", s1, FTPStrError(err));
 				}
 			} else if ((s1 == NULL) || (s1[0] == '\0')) { 
-				(void) fprintf(stderr, "%s: %s.\n", s2, FTPStrError(cip->errNo));
+				FTPLogError(cip, kDontPerror, "%s: %s.\n", s2, FTPStrError(err));
 			} else {
-				(void) fprintf(stderr, "%s %s: %s.\n", s1, s2, FTPStrError(cip->errNo));
+				FTPLogError(cip, kDontPerror, "%s %s: %s.\n", s1, s2, FTPStrError(err));
 			}
 		}
 	}
