@@ -1,11 +1,14 @@
 /* bookmark.c 
  *
- * Copyright (c) 1992-2001 by Mike Gleason.
+ * Copyright (c) 1992-2016 by Mike Gleason.
  * All rights reserved.
  * 
  */
 
 #include "syshdrs.h"
+#ifdef PRAGMA_HDRSTOP
+#	pragma hdrstop
+#endif
 
 #include "bookmark.h"
 #include "util.h"
@@ -52,7 +55,11 @@ BookmarkToURL(BookmarkPtr bmp, char *url, size_t urlsize)
 	}
 	(void) Strncat(url, bmp->name, urlsize);
 	if (bmp->port != 21) {
+#ifdef HAVE_SNPRINTF
+		(void) snprintf(pbuf, sizeof(pbuf) - 1, ":%u", (unsigned int) bmp->port);
+#else
 		(void) sprintf(pbuf, ":%u", (unsigned int) bmp->port);
+#endif
 		(void) Strncat(url, pbuf, urlsize);
 	}
 	if (bmp->dir[0] == '/') {
@@ -119,7 +126,8 @@ HexCharToNibble(int c)
 		case 'E':
 		case 'F':
 			return (c - 'A' + 10);
-
+		default:
+			break;
 	}
 	return (-1);	/* Error. */
 }	/* HexCharToNibble */
@@ -145,7 +153,9 @@ ParseHostLine(char *line, BookmarkPtr bmp)
 
 	SetBookmarkDefaults(bmp);
 	s = line;
-	tokenend = token + sizeof(token) - 1;
+	tokenend = token;
+	tokenend += sizeof(token);
+	--tokenend;
 	result = -1;
 	for (i=1; ; i++) {
 		if (*s == '\0')
@@ -169,7 +179,7 @@ ParseHostLine(char *line, BookmarkPtr bmp)
 				if ((n1 >= 0) && (n2 >= 0)) {
 					n = (n1 << 4) | n2;
 					if (d < tokenend)
-						*(unsigned char *)d++ = (unsigned int) n;
+						*(unsigned char *)d++ = (unsigned char) n;
 				}
 				s += 3;
 			} else {
@@ -200,7 +210,7 @@ ParseHostLine(char *line, BookmarkPtr bmp)
 					bmp->port = (unsigned int) atoi(token);
 				break;
 			case 9:
-				(void) sscanf(token, "%lx", &L);
+				(void) sscanf(token, "%lx", (unsigned long *) &L);
 				bmp->lastCall = (time_t) L;
 				break;
 			case 10: bmp->hasSIZE = atoi(token); break;
@@ -297,7 +307,10 @@ OpenBookmarkFile(int *numBookmarks0)
 		return NULL;		/* Okay to not have one yet. */
 	}
 	
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+#else
 	(void) chmod(pathName, 00600);
+#endif
 	if (FGets(line, sizeof(line), fp) == NULL) {
 		(void) fprintf(stderr, "%s: invalid format.\n", pathName);
 		(void) fclose(fp);
@@ -316,7 +329,11 @@ OpenBookmarkFile(int *numBookmarks0)
 			return NULL;
 		}
 		(void) STRNCPY(path2, pathName);
+#ifdef HAVE_SNPRINTF
+		(void) snprintf(line, sizeof(line) - 1, ".v%d", version);
+#else
 		(void) sprintf(line, ".v%d", version);
+#endif
 		(void) STRNCAT(path2, line);
 		(void) rename(pathName, path2);
 		(void) fprintf(stderr, "%s: old version.\n", pathName);
@@ -399,16 +416,22 @@ GetBookmark(const char *const bmabbr, Bookmark *bmp)
 	int exactMatch = 0;
 	size_t bmabbrLen;
 	char *cp;
+	char bmabbrtrunc[sizeof(bmp->bookmarkName)];
 
 	fp = OpenBookmarkFile(NULL);
 	if (fp == NULL)
 		return (-1);
 
+	memset(&byHostName, 0, sizeof(Bookmark));
+	memset(&byHostAbbr, 0, sizeof(Bookmark));
+	memset(&byBmAbbr, 0, sizeof(Bookmark));
+
+	STRNCPY(bmabbrtrunc, bmabbr);
 	bmabbrLen = strlen(bmabbr);
 	while (FGets(line, sizeof(line), fp) != NULL) {
 		if (ParseHostLine(line, bmp) < 0)
 			continue;
-		if (ISTREQ(bmp->bookmarkName, bmabbr)) {
+		if (ISTREQ(bmp->bookmarkName, bmabbrtrunc)) {
 			/* Exact match, done. */
 			byBmNameFlag = bmabbrLen;
 			exactMatch = 1;
@@ -481,7 +504,7 @@ GetBookmark(const char *const bmabbr, Bookmark *bmp)
 static int
 BookmarkSortProc(const void *a, const void *b)
 {
-	return (ISTRCMP((*(Bookmark *)a).bookmarkName, (*(Bookmark *)b).bookmarkName));	
+	return (ISTRCMP((*(const Bookmark *)a).bookmarkName, (*(const Bookmark *)b).bookmarkName));	
 }	/* BookmarkSortProc */
 
 
@@ -489,7 +512,7 @@ BookmarkSortProc(const void *a, const void *b)
 static int
 BookmarkSearchProc(const void *key, const void *b)
 {
-	return (ISTRCMP((char *) key, (*(Bookmark *)b).bookmarkName));	
+	return (ISTRCMP((const char *) key, (*(const Bookmark *)b).bookmarkName));	
 }	/* BookmarkSearchProc */
 
 
@@ -533,7 +556,7 @@ LoadBookmarkTable(void)
 	if ((nb != gNumBookmarks) && (gBookmarkTable != NULL)) {
 		/* Re-loading the table from disk. */
 		gBookmarkTable = (Bookmark *) realloc(gBookmarkTable, (size_t) (nb + 1) * sizeof(Bookmark));
-		memset(gBookmarkTable, 0, (nb + 1) * sizeof(Bookmark));
+		memset(gBookmarkTable, 0, (size_t) (nb + 1) * sizeof(Bookmark));
 	} else {
 		gBookmarkTable = calloc((size_t) (nb + 1), (size_t) sizeof(Bookmark));
 	}
@@ -574,17 +597,21 @@ BmEscapeTok(char *dst, size_t dsize, char *src)
 			/* These need to be escaped. */
 			if ((dst + 1) < dlim) {
 				*dst++ = '\\';
-				*dst++ = c;
+				*dst++ = (char) c;
 			}
 		} else if (!isprint(c)) {
 			/* Escape non-printing characters. */
 			if ((dst + 2) < dlim) {
-				(void) sprintf(dst, "$%02x", c);
+#ifdef HAVE_SNPRINTF
+				(void) snprintf(dst, (size_t) (dlim - dst), "$%02x", (unsigned int) c);
+#else
+				(void) sprintf(dst, "$%02x", (unsigned int) c);
+#endif
 				dst += 3;
 			}
 		} else {
 			if (dst < dlim)
-				*dst++ = c;
+				*dst++ = (char) c;
 		}
 	}
 	*dst = '\0';
@@ -646,7 +673,14 @@ SwapBookmarkFiles(void)
 
 	(void) OurDirectoryPath(path2, sizeof(path2), kBookmarkFileName);
 	(void) OurDirectoryPath(pathName, sizeof(pathName), kTmpBookmarkFileName);
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__) && !defined(getpid)
+#	define getpid _getpid
+#endif
+#ifdef HAVE_SNPRINTF
+	(void) snprintf(pidStr, sizeof(pidStr) - 1, "-%u.txt", (unsigned int) getpid());
+#else
 	(void) sprintf(pidStr, "-%u.txt", (unsigned int) getpid());
+#endif
 	(void) STRNCAT(pathName, pidStr);
 
 	(void) remove(path2);
@@ -674,7 +708,11 @@ OpenTmpBookmarkFile(int nb)
 
 	(void) OurDirectoryPath(path2, sizeof(path2), kBookmarkFileName);
 	(void) OurDirectoryPath(pathName, sizeof(pathName), kTmpBookmarkFileName);
+#ifdef HAVE_SNPRINTF
+	(void) snprintf(pidStr, sizeof(pidStr) - 1, "-%u.txt", (unsigned int) getpid());
+#else
 	(void) sprintf(pidStr, "-%u.txt", (unsigned int) getpid());
+#endif
 	(void) STRNCAT(pathName, pidStr);
 
 	outfp = fopen(pathName, FOPEN_WRITE_TEXT);
@@ -683,7 +721,10 @@ OpenTmpBookmarkFile(int nb)
 		perror(pathName);
 		return (NULL);
 	}
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+#else
 	(void) chmod(pathName, 00600);
+#endif
 	if (nb > 0) {
 		if (fprintf(outfp, "NcFTP bookmark-file version: %d\nNumber of bookmarks: %d\n", kBookmarkVersion, nb) < 0) {
 			(void) fprintf(stderr, "Could not save bookmark.\n");
@@ -843,3 +884,42 @@ DefaultBookmarkName(char *dst, size_t siz, char *src)
 	}
 	(void) Strncpy(dst, token, siz);
 }	/* DefaultBookmarkName */
+
+
+
+
+int
+AddNewItemToBookmarkTable(void)
+{
+	int nb;
+	BookmarkPtr newTable, bmp;
+
+	if (gBookmarkTable == NULL)
+		return (-1);
+
+	nb = gNumBookmarks + 1;
+	
+	newTable = (BookmarkPtr) realloc(gBookmarkTable, (size_t) (nb) * sizeof(Bookmark));
+	if (newTable == NULL)
+		return (-1);
+
+	gBookmarkTable = newTable;
+	gNumBookmarks = nb;
+
+	bmp = &newTable[nb - 1];
+	SetBookmarkDefaults(bmp);
+
+	return (nb - 1);
+}	/* AddNewItemToBookmarkTable */
+
+
+
+
+void
+DisposeBookmarkTable(void)
+{
+	if (gBookmarkTable != NULL) {
+		free(gBookmarkTable);
+		gBookmarkTable = NULL;
+	}
+}	/* DisposeBookmarkTable */
