@@ -1,11 +1,14 @@
 /* pref.c
  *
- * Copyright (c) 1992-2001 by Mike Gleason.
+ * Copyright (c) 1992-2016 by Mike Gleason.
  * All rights reserved.
  * 
  */
 
 #include "syshdrs.h"
+#ifdef PRAGMA_HDRSTOP
+#	pragma hdrstop
+#endif
 
 #ifdef ncftp
 #include "progress.h"
@@ -68,7 +71,7 @@ int gPrefsDirty = 0;
 
 extern FTPLibraryInfo gLib;
 extern FTPConnectionInfo gConn;
-extern char gOurDirectoryPath[], gUser[], gVersion[];
+extern char gOurDirectoryPath[];
 
 PrefOpt gPrefOpts[] = {
 	{ "anonopen",				PREFOBSELETE },
@@ -115,9 +118,8 @@ int gNumPrefOpts = ((int)(sizeof(gPrefOpts) / sizeof(PrefOpt)) - 1);
 
 
 void
-SetAnonPass(int UNUSED(t), const char *const val, FILE *const fp)
+SetAnonPass(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%s", gLib.defaultAnonPassword);
 	} else {
@@ -128,9 +130,8 @@ SetAnonPass(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetAutoAscii(int UNUSED(t), const char *const val, FILE *const fp)
+SetAutoAscii(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%s", gAutoAscii);
 	} else {
@@ -146,9 +147,8 @@ SetAutoAscii(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetAutoResume(int UNUSED(t), const char *const val, FILE *const fp)
+SetAutoResume(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%s", YESNO(gAutoResume));
 	} else {
@@ -159,9 +159,8 @@ SetAutoResume(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetAutoSaveChangesToExistingBookmarks(int UNUSED(t), const char *const val, FILE *const fp)
+SetAutoSaveChangesToExistingBookmarks(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%s", YESNO(gAutoSaveChangesToExistingBookmarks));
 	} else {
@@ -172,9 +171,8 @@ SetAutoSaveChangesToExistingBookmarks(int UNUSED(t), const char *const val, FILE
 
 
 void
-SetConfirmClose(int UNUSED(t), const char *const val, FILE *const fp)
+SetConfirmClose(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%s", YESNO(gConfirmClose));
 	} else {
@@ -185,35 +183,34 @@ SetConfirmClose(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetConnTimeout(int UNUSED(t), const char *const val, FILE *const fp)
+SetConnTimeout(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%d", gConnTimeout);
 	} else {
-		gConn.connTimeout = gConnTimeout = atoi(val);
+		gConn.connTimeout = (unsigned int) atoi(val);
+        gConnTimeout = atoi(val);
 	}
 }	/* SetConnTimeout */
 
 
 
 void
-SetCtrlTimeout(int UNUSED(t), const char *const val, FILE *const fp)
+SetCtrlTimeout(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%d", gCtrlTimeout);
 	} else {
-		gConn.ctrlTimeout = gCtrlTimeout = atoi(val);
+        gConn.ctrlTimeout = (unsigned int) atoi(val);
+        gCtrlTimeout = atoi(val);
 	}
 }	/* SetCtrlTimeout */
 
 
 
 void
-SetLogSize(int UNUSED(t), const char *const val, FILE *const fp)
+SetLogSize(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%d", gMaxLogSize);
 	} else {
@@ -224,9 +221,8 @@ SetLogSize(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetNoAds(int UNUSED(t), const char *const val, FILE *const fp)
+SetNoAds(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%s", YESNO(gDoNotDisplayAds));
 	} else {
@@ -237,9 +233,8 @@ SetNoAds(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetOneTimeMessages(int UNUSED(t), const char *const val, FILE *const fp)
+SetOneTimeMessages(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%s", gOneTimeMessagesSeen);
 	} else {
@@ -250,9 +245,8 @@ SetOneTimeMessages(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetPager(int UNUSED(t), const char *const val, FILE *const fp)
+SetPager(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%s", gPager);
 	} else {
@@ -263,11 +257,10 @@ SetPager(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetPassive(int UNUSED(t), const char *const val, FILE *const fp)
+SetPassive(const char *const val, FILE *const fp)
 {
 	int m;
 
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		m = (gFwDataPortMode >= 0) ? gFwDataPortMode : gDataPortMode;
 		if (m == kSendPortMode) {
@@ -286,7 +279,7 @@ SetPassive(int UNUSED(t), const char *const val, FILE *const fp)
 			gDataPortMode = kFallBackToSendPortMode;
 		else if (ISTREQ(val, "on"))
 			gDataPortMode = kPassiveMode;
-		else if ((int) isdigit(val[0]))
+		else if (isdigit((int) val[0]))
 			gDataPortMode = atoi(val);
 		else
 			gDataPortMode = kSendPortMode;
@@ -298,9 +291,8 @@ SetPassive(int UNUSED(t), const char *const val, FILE *const fp)
 
 #ifdef ncftp
 void
-SetProgressMeter(int UNUSED(t), const char *const val, FILE *const fp)
+SetProgressMeter(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		if (gProgressMeter == PrStatBar) {
 			(void) fprintf(fp, "%s", "2 (statbar)");
@@ -321,9 +313,8 @@ SetProgressMeter(int UNUSED(t), const char *const val, FILE *const fp)
 }	/* SetProgressMeter */
 #else
 void
-SetProgressMeter(int UNUSED(t), const char *const UNUSED(val), FILE *const UNUSED(fp))
+SetProgressMeter(const char *const UNUSED(val), FILE *const UNUSED(fp))
 {
-	LIBNCFTP_USE_VAR(t);
 	LIBNCFTP_USE_VAR(val);
 	LIBNCFTP_USE_VAR(fp);
 }	/* SetProgressMeter */
@@ -332,11 +323,10 @@ SetProgressMeter(int UNUSED(t), const char *const UNUSED(val), FILE *const UNUSE
 
 
 void
-SetRedialDelay(int UNUSED(t), const char *const val, FILE *const fp)
+SetRedialDelay(const char *const val, FILE *const fp)
 {
 	int i;
 
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%d", gRedialDelay);
 	} else {
@@ -350,9 +340,8 @@ SetRedialDelay(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetSavePasswords(int UNUSED(t), const char *const val, FILE *const fp)
+SetSavePasswords(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		if (gSavePasswords < 0)
 			(void) fprintf(fp, "%s", "ask");
@@ -369,15 +358,15 @@ SetSavePasswords(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetSOBufsize(int UNUSED(t), const char *const val, FILE *const fp)
+SetSOBufsize(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%d", gSOBufsize);
 		if (gSOBufsize <= 0)
 			(void) fprintf(fp, "%s", " (use system default)");
 	} else {
-		gConn.dataSocketRBufSize = gConn.dataSocketSBufSize = gSOBufsize = atoi(val);
+		gConn.dataSocketRBufSize = gConn.dataSocketSBufSize = (size_t) atoi(val);
+        gSOBufsize = atoi(val);
 	}
 }	/* SetSOBufsize */
 
@@ -385,22 +374,21 @@ SetSOBufsize(int UNUSED(t), const char *const val, FILE *const fp)
 
 
 void
-SetXferTimeout(int UNUSED(t), const char *const val, FILE *const fp)
+SetXferTimeout(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%d", gXferTimeout);
 	} else {
-		gConn.xferTimeout = gXferTimeout = atoi(val);
+		gConn.xferTimeout = (unsigned int) atoi(val);
+        gXferTimeout = atoi(val);
 	}
 }	/* SetXferTimeout */
 
 
 
 void
-SetXtTitle(int UNUSED(t), const char *const val, FILE *const fp)
+SetXtTitle(const char *const val, FILE *const fp)
 {
-	LIBNCFTP_USE_VAR(t);
 	if (fp != NULL) {
 		(void) fprintf(fp, "%s", YESNO(gMaySetXtermTitle));
 	} else {
@@ -418,7 +406,7 @@ Show1(int t)
 
 	(void) printf("%-30s ", p->varname);
 	if (p->proc != (PrefProc) 0)
-		(*p->proc)(t, NULL, stdout);
+		(*p->proc)(NULL, stdout);
 	(void) printf("\n");
 }	/* Show1 */
 
@@ -459,7 +447,7 @@ Set(const char *const tok1, const char *const tok2)
 				if (gPrefOpts[t].visible == kPrefOptObselete) {
 					(void) printf("The \"%s\" option is obselete or not implemented.\n", tok1);
 				} else if (gPrefOpts[t].proc != (PrefProc) 0) {
-					(*gPrefOpts[t].proc)(t, tok2, NULL);
+					(*gPrefOpts[t].proc)(tok2, NULL);
 					gPrefsDirty++;
 				}
 				break;
@@ -540,7 +528,7 @@ ProcessPrefsFile(FILE *const fp)
 					 */
 					gPrefsDirty++;
 				} else if (gPrefOpts[t].proc != (PrefProc) 0) {
-					(*gPrefOpts[t].proc)(t, tok2, NULL);
+					(*gPrefOpts[t].proc)(tok2, NULL);
 				}
 			}
 		}
@@ -625,13 +613,13 @@ InitPrefs(void)
 	gCtrlTimeout = 135;
 	gDataPortMode = kFallBackToSendPortMode;
 	gConn.dataPortMode = gDataPortMode;
-	gAutoResume = 0;
+	gAutoResume = 1;
 	gSOBufsize = 0;
 	gMaxLogSize = 10240;
-	gConfirmClose = 1;
+	gConfirmClose = 0;
 	gAutoSaveChangesToExistingBookmarks = 0;
 	gRedialDelay = kDefaultRedialDelay;
-	STRNCPY(gAutoAscii, "|.txt|.asc|.html|.htm|.css|.xml|.ini|.sh|.pl|.hqx|.cfg|.c|.h|.cpp|.hpp|.bat|.m3u|.pls|");
+	STRNCPY(gAutoAscii, "|.txt|.asc|.html|.htm|.css|.xml|.ini|.pl|.hqx|.cfg|.c|.h|.cpp|.hpp|.bat|.m3u|.pls|");
 
 	/* PLEASE do not change the default from 0, and please
 	 * don't hack out the portion in main.c which displays
@@ -641,7 +629,8 @@ InitPrefs(void)
 	 */
 	gDoNotDisplayAds = 0;
 
-#if (defined(WIN32) || defined(_WINDOWS)) && defined(_CONSOLE)
+#if ( (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__) )\
+	&& defined(_CONSOLE)
 	gMaySetXtermTitle = 1;
 #else
 	gMaySetXtermTitle = 0;
@@ -705,7 +694,11 @@ SavePrefs(void)
 
 	(void) OurDirectoryPath(pathName, sizeof(pathName), kPrefFileName);
 
+#ifdef HAVE_SNPRINTF
+	(void) snprintf(tName, sizeof(tName) - 1, "tpref%06u.txt", (unsigned int) getpid());
+#else
 	(void) sprintf(tName, "tpref%06u.txt", (unsigned int) getpid());
+#endif
 	(void) OurDirectoryPath(pathName2, sizeof(pathName2), tName);
 
 	fp = fopen(pathName2, FOPEN_WRITE_TEXT);
@@ -716,7 +709,7 @@ SavePrefs(void)
 		for (t=0; t<gNumPrefOpts; t++) {
 			if (gPrefOpts[t].visible != kPrefOptObselete) {
 				(void) fprintf(fp, "%s=", gPrefOpts[t].varname);
-				(*gPrefOpts[t].proc)(t, NULL, fp);
+				(*gPrefOpts[t].proc)(NULL, fp);
 				(void) fprintf(fp, "\n");
 			}
 		}
@@ -725,7 +718,7 @@ SavePrefs(void)
 		if (rename(pathName2, pathName) < 0) {
 			perror("could not finish saving preferences file");
 			(void) unlink(pathName2);
-		};
+		}
 	}
 }	/* SavePrefs */
 
