@@ -283,7 +283,7 @@ WriteSpoolEntry(
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 		if (WinStat64(lpathname, &st) == 0) {
 #else
-		if (lstat(lpathname, &st) == 0) {
+		if (Lstat(lpathname, &st) == 0) {
 #endif
 			if (lfilesize <= 0)
 				lfilesize = (double) st.st_size;
