@@ -5,7 +5,6 @@
  *
  */
 
-
 #include "syshdrs.h"
 #ifdef PRAGMA_HDRSTOP
 #	pragma hdrstop
@@ -144,6 +143,10 @@ void DrawHostList(void)
 	int begy, begx;
 	char spec[32];
 
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wformat-nonliteral"
+#endif
 	getmaxyx(gHostListWin, lmaxy, lmaxx);
 	getbegyx(gHostListWin, begy, begx);
 	getmaxyx(gHostWin, maxy, maxx);
@@ -158,24 +161,34 @@ void DrawHostList(void)
 		gHostWin,
 		begy - 1,
 		begx,
-		strcpy(spec, "%s"),	/* avoid warnings on BSD */
+		STRNCPY(spec, "%s"),	/* avoid warnings on BSD */
 		"Number of bookmarks"
 	);
 	WAttr(gHostWin, kUnderline, 0);
 	wprintw(
 		gHostWin,
-		strcpy(spec, ": %3d"),
+		STRNCPY(spec, ": %3d"),
 		gNumBookmarks
 	);
 
 	if (gHostListWinWide == 0) {
-		sprintf(spec, "%%-16.16s %%-%ds", lmaxx - 17);
+#ifdef HAVE_SNPRINTF
+		(void) snprintf(spec, sizeof(spec) - 1, 
+#else
+		(void) sprintf(spec,
+#endif
+			"%%-16.16s %%-%ds", lmaxx - 17);
 		lastLine = lmaxy + gHostListWinStart;
 		for (i=gHostListWinStart; (i<lastLine) && (i<gNumBookmarks); i++) {
 			rsip = &gBookmarkTable[i];
 			if (rsip == gCurHostListItem)
 				WAttr(gHostListWin, kReverse, 1);
-			sprintf(str, spec, rsip->bookmarkName, rsip->name);
+#ifdef HAVE_SNPRINTF
+			(void) snprintf(str, sizeof(str) - 1, 
+#else
+			(void) sprintf(str,
+#endif
+				spec, rsip->bookmarkName, rsip->name);
 			str[lmaxx] = '\0';
 			DrawStrAt(gHostListWin, i - gHostListWinStart, 0, str);
 			swclrtoeol(gHostListWin);
@@ -190,7 +203,12 @@ void DrawHostList(void)
 			if (rsip == gCurHostListItem)
 				WAttr(gHostListWin, kReverse, 1);
 			BookmarkToURL(rsip, url, sizeof(url));
-			sprintf(str, "%-16.16s  ", rsip->bookmarkName);
+#ifdef HAVE_SNPRINTF
+			(void) snprintf(str, sizeof(str) - 1, 
+#else
+			(void) sprintf(str,
+#endif
+				"%-16.16s  ", rsip->bookmarkName);
 			STRNCAT(str, url);
 			memset(url, 0, sizeof(url));
 			AbbrevStr(url, str, (size_t) lmaxx, 1);
@@ -209,7 +227,12 @@ void DrawHostList(void)
 		swclrtoeol(gHostListWin);
 	}
 	wmove(gHostWin, maxy - 3, 2);
-	sprintf(spec, "%%-%ds", maxx - 4);
+#ifdef HAVE_SNPRINTF
+	(void) snprintf(spec, sizeof(spec) - 1, 
+#else
+	(void) sprintf(spec,
+#endif
+		"%%-%ds", maxx - 4);
 	if (gCurHostListItem == NULL) {
 		str[0] = '\0';
 	} else if (gCurHostListItem->comment[0] == '\0') {
@@ -227,8 +250,11 @@ void DrawHostList(void)
 	wprintw(gHostWin, spec, str);
 	wmove(gHostWin, maxy - 1, 0);
 	UpdateHostWindows(0);
-}	/* DrawHostList */
 
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
+}	/* DrawHostList */
 
 
 
@@ -236,7 +262,7 @@ void DrawHostList(void)
 int HostWinGetKey(void)
 {
 	int c;
-	int uc;
+	unsigned int uc;
 	int escmode;
 	int maxy;
 
@@ -485,7 +511,7 @@ void EditHostWinGetStr(char *dst, size_t size, int canBeEmpty, int canEcho)
 	 * the dst here, which would make it an empty string.
 	 */
 	if ((wgp.changed) || (canBeEmpty == kOkayIfEmpty))
-		strcpy(dst, str);
+		Strncpy(dst, str, size);
 
 	wmove(gEditHostWin, maxy - 1, 0);
 	wclrtoeol(gEditHostWin);
@@ -568,8 +594,12 @@ void EditHostWinDraw(int flags, int hilite)
 	if (hilite)
 		WAttr(gEditHostWin, kReverse, 1);
 	getmaxyx(gEditHostWin, maxy, maxx);
-	sprintf(spec, " %%-26s%%-%ds",
-		maxx - 32);
+#ifdef HAVE_SNPRINTF
+	(void) snprintf(spec, sizeof(spec) - 1, 
+#else
+	(void) sprintf(spec,
+#endif
+		" %%-26s%%-%ds", maxx - 32);
 
 	/* Now draw the items on a case-by-case basis. */
 	if (TESTBIT(flags, kNicknameEditWindowItem)) {
@@ -645,7 +675,12 @@ void EditHostWinDraw(int flags, int hilite)
 		wclrtoeol(gEditHostWin);
 	}
 	if (TESTBIT(flags, kPortEditWindowItem)) {
-		sprintf(str, "%u", (gEditRsi.port == 0) ? 21 : (unsigned int) gEditRsi.port);
+#ifdef HAVE_SNPRINTF
+		(void) snprintf(str, sizeof(str) - 1, 
+#else
+		(void) sprintf(str,
+#endif
+			"%u", (gEditRsi.port == 0) ? 21 : (unsigned int) gEditRsi.port);
 		mvwprintw(gEditHostWin, kPortEditWindowItem + f, 3, spec,
 			"Port:",
 			str
@@ -1086,6 +1121,9 @@ void HostWinGetStr(char *str, size_t size)
 
 /*ARGSUSED*/
 static void
+#if (defined(__GNUC__)) && (__GNUC__ >= 2)
+__attribute__ ((noreturn))
+#endif
 SigIntHostWin(int UNUSED(sig))
 {
 	LIBNCFTP_USE_VAR(sig);
